import java.util.*;
import java.lang.*;
import java.text.*;
import java.io.*;

/**
 * This is a simple class for testing your code.  You might want to
 * extend or modify this class, or write your own.
 **/
public class Test {
    /* won't print this HMM if more than this many states */
    private static final int MAX_STATES = 100;

    /* won't print an evidence table for this HMM if more than this
     * many outputs */
    private static final int MAX_OUTPUTS = 100;

    /**
     * A simple main that loads the dataset contained in the file
     * named in the first command-line argument, builds an HMM, prints
     * it out and finds and prints the most likely state sequence for
     * each of the output sequences.
     **/
    public static void main(String[] argv)
	throws FileNotFoundException, IOException {

	String file_name = "";
	try {
	    file_name = argv[0];
	} catch (Exception e) {
	    System.err.println("Arguments: <file_name>");
	    return;
	}

	DataSet ds = new DataSet(file_name);

	Hmm h = new Hmm(ds.numStates, ds.numOutputs);

	h.buildFromData(ds.trainState, ds.trainOutput);

	if (ds.numStates <= MAX_STATES) {

	    System.out.println("Start probabilities:");
	    for (int s = 0; s < ds.numStates; s++)
		System.out.println(name(ds.stateName[s]) + ": " +
				   num(h.getStartProb(s)));

	    System.out.println();
	    System.out.println("Transition probabilities:");

	    System.out.print(name("") + " ");
	    for (int s = 0; s < ds.numStates; s++)
		System.out.print(" " + name(ds.stateName[s]));

	    System.out.println();
	    for (int s = 0; s < ds.numStates; s++) {
		System.out.print(name(ds.stateName[s]) + ":");
		for (int u = 0; u < ds.numStates; u++) {
		    System.out.print(" " + num(h.getTransProb(s,u)));
		}
		System.out.println();
	    }

	    if (ds.numOutputs <= MAX_OUTPUTS) {

		System.out.println();
		System.out.println("Output probabilities:");

		System.out.print(name("") + " ");
		for (int o = 0; o < ds.numOutputs; o++)
		    System.out.print(" " + name(ds.outputName[o]));

		System.out.println();
		for (int s = 0; s < ds.numStates; s++) {
		    System.out.print(name(ds.stateName[s]) + ":");
		    for (int o = 0; o < ds.numOutputs; o++) {
			System.out.print(" " + num(h.getOutputProb(s,o)));
		    }
		    System.out.println();
		}
	    }
	}

	System.out.println();
	for (int i = 0; i < ds.testOutput.length; i++) {
	    int[] state = h.mostLikelySequence(ds.testOutput[i]);
	    int errors = 0;

	    System.out.println();
	    System.out.println("sequence "+i+":");
	    for (int j = 0; j < state.length; j++) {
		System.out.println(ds.stateName[ds.testState[i][j]]+"\t"+
				   ds.stateName[state[j]] +"\t"+
				   ds.outputName[ds.testOutput[i][j]]);
		if (state[j] != ds.testState[i][j])
		    errors++;
	    }
	    System.out.println("errors: " + errors + " / " + state.length +
			       " = " + ((double) errors)/state.length);
	}



    }

    private static NumberFormat nf = new DecimalFormat("#.000");

    private static String name(String s) {
	return (s + "    ").substring(0, 4);
    }

    private static String num(double d) {
	return nf.format(d);
    }

}
	    
